class Voice(object):
    def __init__(self, id, name=None, languages=[], gender=None, age=None):
        self.id = id
        self.name = name
        self.languages = languages
        self.gender = gender
        self.age = age

    def __str__(self):
        return """<Voice id=%(id)s
          name=%(name)s
          languages=%(languages)s
          gender=%(gender)s
          age=%(age)s>""" % self.__dict__



import os
import time
from traceback import print_exc
from unitts.voice import Voice
from appPublic.background import Background
from text2sentences import text_to_sentences

class BaseDriver:
	def __init__(self, proxy):
		self._proxy = proxy
		self._tts = None
		self.sentences = []
		self.normal_voice = {}
		self.dialog_voice = {}
		self.pitch = 5
		self.language = 'zh'
		self.format = 3
		self.voice = 0
		self.cmds = []
		self._completed = True
		self.running = True
		self.task = None
		self._speaked = False
		self._speaking = False
		self.voices = self.get_voices()

	def get_voices(self):
		return []

	def set_type_voice(self, sentence):
		raise Exception('fixme')

	def backtask(self):
		print('player back task running ...')
		self.running = True
		while self.running:
			self._pump()
			time.sleep(0.01)
		print('player back task end ...')

	def is_speaking(self):
		return self._speaked

	def speak_finish(self):
		self._proxy.notify('finished-sentence')
		self._speaking = False
		self._speaked = False

	def _push(self, cmd):
		self.cmds.append(cmd)

	def isSpeaking(self):
		return self._speaked

	def _pump(self):
		if self._speaking:
			# print('tts driver is busy')
			if not self._speaked:
				self._speaked = self.isSpeaking()
			else:
				if not self.isSpeaking():
					self.speak_finish()
			return False
		if len(self.cmds) < 1:
			# print('No cmd to do')
			if self._proxy.isBusy():
				self._proxy.setBusy(False)
				self._proxy.notify('finished-text')
			return False

		self._speaking = True
		args = self.cmds.pop(0)
		print('start speak', args)
		self.command(*args)
		pos, _ = args
		self._proxy.notify('started-sentence', pos)
		return True

	def destroy(self):
		if self.task:
			self.running = False
			self.task.join()
			self.task = None
	
	def __del__(self):
		self.destroy()

	def is_in_loop(self):
		if self.task:
			return True
		return False

	def startLoop(self, *args):
		print('startLoop() called')
		self._proxy.setBusy(False)
		self.task = Background(self.backtask)
		self.task.start()

	def endLoop(self):
		print('endLoop() called')
		self.cmds = []
		self._proxy.setBusy(False)
		if self.task:
			self.running = False
			self.task.join()
			self.task = None

	def pre_command(self, sentence):
		# return sentence's start_pos and a object to use to product voice
		# self.set_type_voice(sentence)
		return sentence.start_pos, sentence.text
	
	def command(self, *args):
		raise Exception('fixme')

	def say(self, sentence):
		self._proxy.setBusy(False)
		self._completed = True
		args = self.pre_command(sentence)
		self._push(args)

	def stop(self):
		if self._proxy.isBusy():
			self._completed = False

	def getProperty(self, name):
		if name == 'normal_voice':
			return self.normal_voice
		if name == 'dialog_voice':
			return self.dialog_voice

		if name == 'voices':
			return Voices

		if name == 'voice':
			for v in self.voices:
				if v.id == self.voice:
					return v
			return None
		if name == 'rate':
			return self.rate
		if name == 'volume':
			return self.volume
		if name == 'pitch':
			return self.pitch
	
	def setProperty(self, name, value):
		if name == 'normal_voice':
			self.normal_voice = value
		if name == 'dialog_voice':
			self.dialog_voice = value
		if name == 'voice':
			self.voice = value
		if name == 'rate':
			self.rate = value
		if name == 'pitch':
			self.rate = value
		if name == 'language':
			self.language = value
		if name == 'volume':
			self.volume = value




import weakref
from .engine import TTSEngine
from .version import __version__

_activeEngines = weakref.WeakValueDictionary()

def init(driverName):
	try:	
		eng = _activeEngines[driverName]
	except KeyError:
		eng = TTSEngine(driverName)
		_activeEngines[driverName] = eng
	return eng



# -*- coding: utf-8 -*-

try:
	from setuptools import setup
except ImportError:
	from distutils.core import setup

with open('unitts/version.py', 'r') as f:
	x = f.read()
	y = x[x.index("'")+1:]
	z = y[:y.index("'")]
	version = z
with open("README.md", "r") as fh:
    long_description = fh.read()

name = "unitts"
description = "unitts is a api for tts engine"
author = "yumoqing"
email = "yumoqing@gmail.com"

package_data = {}

setup(
	name="unitts",
	version=version,
	
	# uncomment the following lines if you fill them out in release.py
	description=description,
	author=author,
	author_email=email,
   	platforms='any',
	install_requires=[
	],
	packages=[
		"unitts"
	],
	package_data=package_data,
	keywords = [
	],
	url="https://github.com/yumoqing/unitts",
	long_description=long_description,
	long_description_content_type="text/markdown",
	classifiers = [
		'Operating System :: OS Independent',
		'Programming Language :: Python :: 3',
		'License :: OSI Approved :: MIT License',
	],
)



from text2sentences import text_to_sentences
from appPublic.uniqueID import getID
import importlib

def buildDriver(tts, driverName):
	_module = importlib.import_module(driverName)
	return _module.buildDriver(tts)
	
class TTSEngine:
	"""
	Usages:
	"""
	def __init__(self, driverName):
		self.driver = buildDriver(self, driverName)
		self._connects = {}
		self._busy = False

	def setBusy(self, f):
		self._busy = f

	def connect(self, subject, func):
		if subject in ['started-sentence', 'started-word',
						'started-text', 'finished-text']:
			self._connects.update({subject:func})
	
	def disconnect(self, subject):
		if subject in ['started-sentence', 'started-word',
						'started-text', 'finished-text']:
			del self._connects[subject]
		
	def say(self, text, pos=0):
		if self.isBusy():
			print('tts is busy, do not thing')
			return
		self.current_pos = pos
		sentences = text_to_sentences(text)
		self.say_sentences(sentences)

	def say_sentences(self, sentences):
		for s in sentences:
			self.driver.say(s)
		
	def notify(self, subject, *args, **kw):
		for s,f in self._connects.items():
			if s == subject:
				f(*args, **kw)

	def isBusy(self):
		return self._busy


	def stop(self):
		pass

	def pause(self):
		pass

	def resume(self):
		pass

	def startLoop(self):
		self.driver.startLoop()

	def is_in_loop(self):
		return self.driver.is_in_loop()

	def endLoop(self):
		self.driver.endLoop()
	
	def getProperty(self, name):
		return self.driver.getProperty(name)

	def setProperty(self, name, value):
		self.driver.setProperty(name, value)

